# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .workflow_runs_create_batch_request_inputs_item import WorkflowRunsCreateBatchRequestInputsItemParams
    from .workflow_runs_create_batch_request_inputs_item_file import WorkflowRunsCreateBatchRequestInputsItemFileParams
    from .workflow_runs_create_batch_response import WorkflowRunsCreateBatchResponseParams
    from .workflow_runs_create_request_file import WorkflowRunsCreateRequestFileParams
    from .workflow_runs_create_request_outputs_item import WorkflowRunsCreateRequestOutputsItemParams
    from .workflow_runs_delete_response import WorkflowRunsDeleteResponseParams
    from .workflow_runs_list_response import WorkflowRunsListResponseParams
_dynamic_imports: typing.Dict[str, str] = {
    "WorkflowRunsCreateBatchRequestInputsItemFileParams": ".workflow_runs_create_batch_request_inputs_item_file",
    "WorkflowRunsCreateBatchRequestInputsItemParams": ".workflow_runs_create_batch_request_inputs_item",
    "WorkflowRunsCreateBatchResponseParams": ".workflow_runs_create_batch_response",
    "WorkflowRunsCreateRequestFileParams": ".workflow_runs_create_request_file",
    "WorkflowRunsCreateRequestOutputsItemParams": ".workflow_runs_create_request_outputs_item",
    "WorkflowRunsDeleteResponseParams": ".workflow_runs_delete_response",
    "WorkflowRunsListResponseParams": ".workflow_runs_list_response",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "WorkflowRunsCreateBatchRequestInputsItemFileParams",
    "WorkflowRunsCreateBatchRequestInputsItemParams",
    "WorkflowRunsCreateBatchResponseParams",
    "WorkflowRunsCreateRequestFileParams",
    "WorkflowRunsCreateRequestOutputsItemParams",
    "WorkflowRunsDeleteResponseParams",
    "WorkflowRunsListResponseParams",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class WorkflowRunStatus(enum.StrEnum):
    """
    The status of a workflow run:
    * `"PENDING"` - The workflow run is waiting to be processed
    * `"PROCESSING"` - The workflow run is currently processing
    * `"NEEDS_REVIEW"` - The workflow run requires manual review
    * `"REJECTED"` - The workflow run was rejected during review
    * `"PROCESSED"` - The workflow run completed successfully
    * `"FAILED"` - The workflow run encountered an error
    * `"CANCELLED"` - The workflow run was cancelled
    * `"CANCELLING"` - The workflow run is being cancelled
    """

    PENDING = "PENDING"
    PROCESSING = "PROCESSING"
    NEEDS_REVIEW = "NEEDS_REVIEW"
    REJECTED = "REJECTED"
    PROCESSED = "PROCESSED"
    FAILED = "FAILED"
    CANCELLED = "CANCELLED"
    CANCELLING = "CANCELLING"
    _UNKNOWN = "__WORKFLOWRUNSTATUS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "WorkflowRunStatus":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        pending: typing.Callable[[], T_Result],
        processing: typing.Callable[[], T_Result],
        needs_review: typing.Callable[[], T_Result],
        rejected: typing.Callable[[], T_Result],
        processed: typing.Callable[[], T_Result],
        failed: typing.Callable[[], T_Result],
        cancelled: typing.Callable[[], T_Result],
        cancelling: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is WorkflowRunStatus.PENDING:
            return pending()
        if self is WorkflowRunStatus.PROCESSING:
            return processing()
        if self is WorkflowRunStatus.NEEDS_REVIEW:
            return needs_review()
        if self is WorkflowRunStatus.REJECTED:
            return rejected()
        if self is WorkflowRunStatus.PROCESSED:
            return processed()
        if self is WorkflowRunStatus.FAILED:
            return failed()
        if self is WorkflowRunStatus.CANCELLED:
            return cancelled()
        if self is WorkflowRunStatus.CANCELLING:
            return cancelling()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import typing_extensions
from ..core.serialization import FieldMetadata
from ..types.webhook_subscription_event_type import WebhookSubscriptionEventType
from ..types.webhook_subscription_resource_type import WebhookSubscriptionResourceType


class WebhookSubscriptionParams(typing_extensions.TypedDict):
    object: typing.Literal["webhook_subscription"]
    id: str
    """
    Unique identifier for the webhook subscription.
    """

    webhook_endpoint_id: typing_extensions.Annotated[str, FieldMetadata(alias="webhookEndpointId")]
    """
    The ID of the webhook endpoint this subscription belongs to.
    """

    resource_type: typing_extensions.Annotated[WebhookSubscriptionResourceType, FieldMetadata(alias="resourceType")]
    resource_id: typing_extensions.Annotated[str, FieldMetadata(alias="resourceId")]
    """
    The ID of the resource this subscription is scoped to (e.g., an extractor, classifier, splitter, or workflow).
    """

    enabled_events: typing_extensions.Annotated[
        typing.Sequence[WebhookSubscriptionEventType], FieldMetadata(alias="enabledEvents")
    ]
    """
    The list of event types this subscription is subscribed to. The valid event types depend on the `resourceType`.
    """

    created_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="createdAt")]
    """
    The date and time the webhook subscription was created.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..core.serialization import FieldMetadata


class ParseConfigBlockOptionsFiguresParams(typing_extensions.TypedDict):
    """
    Options for figure blocks.
    """

    enabled: typing_extensions.NotRequired[bool]
    """
    Whether to include figures in the output.
    """

    figure_image_clipping_enabled: typing_extensions.NotRequired[
        typing_extensions.Annotated[bool, FieldMetadata(alias="figureImageClippingEnabled")]
    ]
    """
    Whether to clip and extract images from figures.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..core.serialization import FieldMetadata


class LegacyProcessorRunMergedProcessorsItemParams(typing_extensions.TypedDict):
    processor_id: typing_extensions.Annotated[str, FieldMetadata(alias="processorId")]
    """
    The ID of the merged processor.
    
    Example: `"ex_Xj8mK2pL9nR4vT7qY5wZ"`
    """

    processor_version_id: typing_extensions.Annotated[str, FieldMetadata(alias="processorVersionId")]
    """
    The ID of the specific processor version used.
    """

    processor_name: typing_extensions.Annotated[str, FieldMetadata(alias="processorName")]
    """
    The name of the merged processor.
    
    Example: `"Invoice Line Items Processor"`
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..core.serialization import FieldMetadata


class FileContentsSheetsItemParams(typing_extensions.TypedDict):
    sheet_name: typing_extensions.Annotated[str, FieldMetadata(alias="sheetName")]
    """
    The name of the sheet.
    """

    raw_text: typing_extensions.NotRequired[typing_extensions.Annotated[str, FieldMetadata(alias="rawText")]]
    """
    The raw text content of the sheet in CSV format.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel


class ParseRunMetrics(UncheckedBaseModel):
    """
    Metrics about the parsing process.

    **Availability:** Present when `status` is `"PROCESSED"`.
    """

    processing_time_ms: typing_extensions.Annotated[float, FieldMetadata(alias="processingTimeMs")] = pydantic.Field(
        alias="processingTimeMs"
    )
    """
    The time taken to process the document in milliseconds.
    """

    page_count: typing_extensions.Annotated[float, FieldMetadata(alias="pageCount")] = pydantic.Field(alias="pageCount")
    """
    The number of pages from the document that were parsed. For PDF and image files, this is the actual page count. For spreadsheet files (Excel, CSV), this is an estimated page count based on content volume (~1000 tokens per page).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .legacy_json_output_metadata_value_citations_item_polygon_item import (
    LegacyJsonOutputMetadataValueCitationsItemPolygonItem,
)


class LegacyJsonOutputMetadataValueCitationsItem(UncheckedBaseModel):
    page: typing.Optional[float] = pydantic.Field(default=None)
    """
    Page number where the citation was found
    """

    reference_text: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="referenceText")] = (
        pydantic.Field(alias="referenceText", default=None)
    )
    """
    The text that was referenced
    """

    polygon: typing.Optional[typing.List[LegacyJsonOutputMetadataValueCitationsItemPolygonItem]] = pydantic.Field(
        default=None
    )
    """
    Array of points defining the polygon around the referenced text
    """

    page_width: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="pageWidth")] = pydantic.Field(
        alias="pageWidth", default=None
    )
    """
    Width of the page in points
    """

    page_height: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="pageHeight")] = (
        pydantic.Field(alias="pageHeight", default=None)
    )
    """
    Height of the page in points
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..core.serialization import FieldMetadata
from .split_run import SplitRunParams


class SplitResultParams(typing_extensions.TypedDict):
    """
    The result of a split step.
    """

    split_run: typing_extensions.Annotated[SplitRunParams, FieldMetadata(alias="splitRun")]
    """
    The split run that was executed.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..core.serialization import FieldMetadata
from ..types.created_at import CreatedAt
from ..types.updated_at import UpdatedAt
from .evaluation_set_entity import EvaluationSetEntityParams


class EvaluationSetParams(typing_extensions.TypedDict):
    object: typing.Literal["evaluation_set"]
    """
    The type of object. In this case, it will always be `"evaluation_set"`.
    """

    id: str
    """
    The ID of the evaluation set.
    
    Example: `"ev_2LcgeY_mp2T5yPaEuq5Lw"`
    """

    name: str
    """
    The name of the evaluation set.
    
    Example: `"Invoice Processing Test Set"`
    """

    description: str
    """
    A description of the evaluation set.
    
    Example: `"Q4 2023 vendor invoices for accuracy testing"`
    """

    entity: EvaluationSetEntityParams
    """
    The extractor, classifier, or splitter associated with this evaluation set.
    """

    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")]
    updated_at: typing_extensions.Annotated[UpdatedAt, FieldMetadata(alias="updatedAt")]



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class LegacyProvidedJsonOutputParams(typing_extensions.TypedDict):
    """
    The output from a JSON Schema processor
    """

    value: typing.Dict[str, typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .parse_config_chunking_strategy_options import ParseConfigChunkingStrategyOptions
from .parse_config_chunking_strategy_type import ParseConfigChunkingStrategyType


class ParseConfigChunkingStrategy(UncheckedBaseModel):
    """
    Strategy for dividing the document into chunks.
    """

    type: typing.Optional[ParseConfigChunkingStrategyType] = pydantic.Field(default=None)
    """
    The type of chunking strategy. Supported values:
    
    * `page`: Chunk document by pages.
    * `document`: Entire document is a single chunk. Essentially no chunking.
    * `section`: Split by logical sections. Not supported for target=spatial.
    """

    options: typing.Optional[ParseConfigChunkingStrategyOptions] = pydantic.Field(default=None)
    """
    Additional options for the chunking strategy.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..core.serialization import FieldMetadata
from .parse_config_block_options_text_agentic import ParseConfigBlockOptionsTextAgenticParams


class ParseConfigBlockOptionsTextParams(typing_extensions.TypedDict):
    """
    Options for text blocks.
    """

    signature_detection_enabled: typing_extensions.NotRequired[
        typing_extensions.Annotated[bool, FieldMetadata(alias="signatureDetectionEnabled")]
    ]
    """
    Whether an additional vision model will be utilized for advanced signature detection. Recommended for most use cases, but should be disabled if signature detection is not necessary and latency is a concern.
    """

    agentic: typing_extensions.NotRequired[ParseConfigBlockOptionsTextAgenticParams]
    """
    Options for agentic text processing using VLM-based review and correction.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .split_run import SplitRun


class SplitResult(UncheckedBaseModel):
    """
    The result of a split step.
    """

    split_run: typing_extensions.Annotated[SplitRun, FieldMetadata(alias="splitRun")] = pydantic.Field(alias="splitRun")
    """
    The split run that was executed.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .api_version_enum import ApiVersionEnum
from .webhook_advanced_options import WebhookAdvancedOptions
from .webhook_endpoint_event_type import WebhookEndpointEventType
from .webhook_endpoint_status import WebhookEndpointStatus


class WebhookEndpoint(UncheckedBaseModel):
    object: typing.Literal["webhook_endpoint"] = "webhook_endpoint"
    id: str = pydantic.Field()
    """
    Unique identifier for the webhook endpoint.
    """

    url: str = pydantic.Field()
    """
    The URL that webhook events are sent to.
    """

    name: str = pydantic.Field()
    """
    A human-readable name for the webhook endpoint.
    """

    status: WebhookEndpointStatus
    enabled_events: typing_extensions.Annotated[
        typing.List[WebhookEndpointEventType], FieldMetadata(alias="enabledEvents")
    ] = pydantic.Field(alias="enabledEvents")
    """
    The list of event types this endpoint is subscribed to at the global level. These are events that are not scoped to a specific resource.
    """

    api_version: typing_extensions.Annotated[ApiVersionEnum, FieldMetadata(alias="apiVersion")] = pydantic.Field(
        alias="apiVersion"
    )
    advanced_options: typing_extensions.Annotated[
        typing.Optional[WebhookAdvancedOptions], FieldMetadata(alias="advancedOptions")
    ] = pydantic.Field(alias="advancedOptions", default=None)
    """
    Advanced configuration options for the webhook endpoint, including custom headers and payload delivery settings.
    """

    created_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="createdAt")] = pydantic.Field(
        alias="createdAt"
    )
    """
    The date and time the webhook endpoint was created.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class WebhookSubscriptionResourceType(enum.StrEnum):
    """
    The type of resource the webhook subscription is for.
    """

    EXTRACTOR = "extractor"
    CLASSIFIER = "classifier"
    SPLITTER = "splitter"
    WORKFLOW = "workflow"
    _UNKNOWN = "__WEBHOOKSUBSCRIPTIONRESOURCETYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "WebhookSubscriptionResourceType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        extractor: typing.Callable[[], T_Result],
        classifier: typing.Callable[[], T_Result],
        splitter: typing.Callable[[], T_Result],
        workflow: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is WebhookSubscriptionResourceType.EXTRACTOR:
            return extractor()
        if self is WebhookSubscriptionResourceType.CLASSIFIER:
            return classifier()
        if self is WebhookSubscriptionResourceType.SPLITTER:
            return splitter()
        if self is WebhookSubscriptionResourceType.WORKFLOW:
            return workflow()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ...core.serialization import FieldMetadata
from ...requests.legacy_processor_version import LegacyProcessorVersionParams


class ProcessorVersionCreateResponseParams(typing_extensions.TypedDict):
    success: bool
    processor_version: typing_extensions.Annotated[
        LegacyProcessorVersionParams, FieldMetadata(alias="processorVersion")
    ]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .classifications import Classifications
from .parse_config import ParseConfig
from .split_advanced_options import SplitAdvancedOptions
from .split_base_processor import SplitBaseProcessor


class SplitOverrideConfig(UncheckedBaseModel):
    """
    Partial configuration override for an existing splitter. All fields are optional — only the fields you provide will override the splitter's saved configuration.
    For example, you can pass only `splitRules` without providing `splitClassifications`.
    """

    base_processor: typing_extensions.Annotated[
        typing.Optional[SplitBaseProcessor], FieldMetadata(alias="baseProcessor")
    ] = pydantic.Field(alias="baseProcessor", default=None)
    base_version: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="baseVersion")] = (
        pydantic.Field(alias="baseVersion", default=None)
    )
    """
    The version of the `"splitting_performance"` or `"splitting_light"` processor to use. If not provided, the latest stable version for the selected `baseProcessor` will be used automatically. See [Splitting Changelog](https://docs.extend.ai/2026-02-09/changelog/splitting/splitting-performance) for more details.
    """

    split_classifications: typing_extensions.Annotated[
        typing.Optional[Classifications], FieldMetadata(alias="splitClassifications")
    ] = pydantic.Field(alias="splitClassifications", default=None)
    split_rules: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="splitRules")] = pydantic.Field(
        alias="splitRules", default=None
    )
    """
    Custom rules to guide the document splitting process in natural language.
    """

    advanced_options: typing_extensions.Annotated[
        typing.Optional[SplitAdvancedOptions], FieldMetadata(alias="advancedOptions")
    ] = pydantic.Field(alias="advancedOptions", default=None)
    """
    Advanced configuration options.
    """

    parse_config: typing_extensions.Annotated[typing.Optional[ParseConfig], FieldMetadata(alias="parseConfig")] = (
        pydantic.Field(alias="parseConfig", default=None)
    )
    """
    Configuration options for the parsing process.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..core.serialization import FieldMetadata
from .split_run import SplitRunParams


class SplitRunFailedWebhookEventParams(typing_extensions.TypedDict):
    event_id: typing_extensions.Annotated[str, FieldMetadata(alias="eventId")]
    """
    Unique identifier for the event
    """

    payload: SplitRunParams



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..core.serialization import FieldMetadata
from ..types.created_at import CreatedAt
from .extract_config import ExtractConfigParams


class ExtractorVersionParams(typing_extensions.TypedDict):
    object: typing.Literal["extractor_version"]
    """
    The type of object. Will always be `"extractor_version"`.
    """

    id: str
    """
    The unique identifier for this version of the extractor.
    
    Example: `"exv_xK9mLPqRtN3vS8wF5hB2cQ"`
    """

    description: typing.Optional[str]
    """
    A description of this version of the extractor.
    
    Example: `"Updated extraction fields for new invoice format"`
    """

    version: str
    """
    The version number or identifier for this specific version of the extractor. The draft version will have version="draft".
    
    Examples: `"1.0"`, `"2.1"`, `"draft"`
    """

    config: ExtractConfigParams
    """
    The configuration settings for this version of the extractor. This is a union of two possible shapes:
    
    - **[JSON Schema config](https://docs.extend.ai/2026-02-09/product/extraction/schema):** The current config format. All extractors created through this API version use this shape.
    - **[Legacy config](https://docs.extend.ai/2025-04-21/product/legacy/legacy-schema):** A fields-array config from a previous API version. This shape is only returned for extractors that were originally configured with the legacy format. This API version does not support creating extractors with legacy configs.
    """

    extractor_id: typing_extensions.Annotated[str, FieldMetadata(alias="extractorId")]
    """
    The ID of the extractor that this version belongs to.
    
    Example: `"ex_Xj8mK2pL9nR4vT7qY5wZ"`
    """

    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError as core_api_error_ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.unchecked_base_model import construct_type
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.internal_server_error import InternalServerError
from ..errors.not_found_error import NotFoundError
from ..errors.payment_required_error import PaymentRequiredError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..requests.classify_config import ClassifyConfigParams
from ..types.api_error import ApiError as types_api_error_ApiError
from ..types.classifier import Classifier
from ..types.max_page_size import MaxPageSize
from ..types.next_page_token import NextPageToken
from ..types.sort_by import SortBy
from ..types.sort_dir import SortDir
from .types.classifiers_list_response import ClassifiersListResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawClassifiersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        next_page_token: typing.Optional[NextPageToken] = None,
        max_page_size: typing.Optional[MaxPageSize] = None,
        sort_by: typing.Optional[SortBy] = None,
        sort_dir: typing.Optional[SortDir] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ClassifiersListResponse]:
        """
        List all classifiers.

        Returns a summary of each classifier. Use `GET /classifiers/{id}` to retrieve the full object including `draftVersion`.

        Parameters
        ----------
        next_page_token : typing.Optional[NextPageToken]

        max_page_size : typing.Optional[MaxPageSize]

        sort_by : typing.Optional[SortBy]

        sort_dir : typing.Optional[SortDir]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ClassifiersListResponse]
            Successfully retrieved classifiers
        """
        _response = self._client_wrapper.httpx_client.request(
            "classifiers",
            method="GET",
            params={
                "nextPageToken": next_page_token,
                "maxPageSize": max_page_size,
                "sortBy": sort_by,
                "sortDir": sort_dir,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ClassifiersListResponse,
                    construct_type(
                        type_=ClassifiersListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    def create(
        self,
        *,
        name: str,
        clone_classifier_id: typing.Optional[str] = OMIT,
        config: typing.Optional[ClassifyConfigParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Classifier]:
        """
        Create a new classifier.

        Parameters
        ----------
        name : str
            The name of the classifier.

        clone_classifier_id : typing.Optional[str]
            The ID of an existing classifier to clone. If provided, the new classifier will be created with the same config as the classifier with this ID. Cannot be provided together with `config`.

            Example: `"cl_BMdfq_yWM3sT-ZzvCnA3f"`

        config : typing.Optional[ClassifyConfigParams]
            The configuration for the classifier. Cannot be provided together with `cloneClassifierId`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Classifier]
            Classifier created successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            "classifiers",
            method="POST",
            json={
                "name": name,
                "cloneClassifierId": clone_classifier_id,
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=ClassifyConfigParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Classifier,
                    construct_type(
                        type_=Classifier,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    def retrieve(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[Classifier]:
        """
        Get details of a classifier.

        Parameters
        ----------
        id : str
            The ID of the classifier to get.

            Example: `"cl_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Classifier]
            Classifier details
        """
        _response = self._client_wrapper.httpx_client.request(
            f"classifiers/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Classifier,
                    construct_type(
                        type_=Classifier,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        config: typing.Optional[ClassifyConfigParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Classifier]:
        """
        Update an existing classifier.

        Parameters
        ----------
        id : str
            The ID of the classifier to update.

            Example: `"cl_Xj8mK2pL9nR4vT7qY5wZ"`

        name : typing.Optional[str]
            The new name of the classifier.

        config : typing.Optional[ClassifyConfigParams]
            The new configuration for the classifier. This will update the draft version of the classifier.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Classifier]
            Classifier updated successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            f"classifiers/{jsonable_encoder(id)}",
            method="POST",
            json={
                "name": name,
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=ClassifyConfigParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Classifier,
                    construct_type(
                        type_=Classifier,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )


class AsyncRawClassifiersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        next_page_token: typing.Optional[NextPageToken] = None,
        max_page_size: typing.Optional[MaxPageSize] = None,
        sort_by: typing.Optional[SortBy] = None,
        sort_dir: typing.Optional[SortDir] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ClassifiersListResponse]:
        """
        List all classifiers.

        Returns a summary of each classifier. Use `GET /classifiers/{id}` to retrieve the full object including `draftVersion`.

        Parameters
        ----------
        next_page_token : typing.Optional[NextPageToken]

        max_page_size : typing.Optional[MaxPageSize]

        sort_by : typing.Optional[SortBy]

        sort_dir : typing.Optional[SortDir]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ClassifiersListResponse]
            Successfully retrieved classifiers
        """
        _response = await self._client_wrapper.httpx_client.request(
            "classifiers",
            method="GET",
            params={
                "nextPageToken": next_page_token,
                "maxPageSize": max_page_size,
                "sortBy": sort_by,
                "sortDir": sort_dir,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ClassifiersListResponse,
                    construct_type(
                        type_=ClassifiersListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    async def create(
        self,
        *,
        name: str,
        clone_classifier_id: typing.Optional[str] = OMIT,
        config: typing.Optional[ClassifyConfigParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Classifier]:
        """
        Create a new classifier.

        Parameters
        ----------
        name : str
            The name of the classifier.

        clone_classifier_id : typing.Optional[str]
            The ID of an existing classifier to clone. If provided, the new classifier will be created with the same config as the classifier with this ID. Cannot be provided together with `config`.

            Example: `"cl_BMdfq_yWM3sT-ZzvCnA3f"`

        config : typing.Optional[ClassifyConfigParams]
            The configuration for the classifier. Cannot be provided together with `cloneClassifierId`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Classifier]
            Classifier created successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            "classifiers",
            method="POST",
            json={
                "name": name,
                "cloneClassifierId": clone_classifier_id,
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=ClassifyConfigParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Classifier,
                    construct_type(
                        type_=Classifier,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    async def retrieve(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Classifier]:
        """
        Get details of a classifier.

        Parameters
        ----------
        id : str
            The ID of the classifier to get.

            Example: `"cl_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Classifier]
            Classifier details
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"classifiers/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Classifier,
                    construct_type(
                        type_=Classifier,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        config: typing.Optional[ClassifyConfigParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Classifier]:
        """
        Update an existing classifier.

        Parameters
        ----------
        id : str
            The ID of the classifier to update.

            Example: `"cl_Xj8mK2pL9nR4vT7qY5wZ"`

        name : typing.Optional[str]
            The new name of the classifier.

        config : typing.Optional[ClassifyConfigParams]
            The new configuration for the classifier. This will update the draft version of the classifier.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Classifier]
            Classifier updated successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"classifiers/{jsonable_encoder(id)}",
            method="POST",
            json={
                "name": name,
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=ClassifyConfigParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Classifier,
                    construct_type(
                        type_=Classifier,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from .insight import InsightParams


class ClassifyOutputParams(typing_extensions.TypedDict):
    id: str
    """
    The unique identifier for this classification
    """

    type: str
    """
    The type of classification
    """

    confidence: float
    """
    A value between 0 and 1 indicating the model's confidence in the classification, where 1 represents maximum confidence
    """

    insights: typing.Sequence[InsightParams]
    """
    Additional insights about the classification decision
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class InsightParams(typing_extensions.TypedDict):
    type: typing.Literal["reasoning"]
    """
    The type of insight. Will always be `"reasoning"` for now.
    """

    content: str
    """
    The content of the reasoning insight.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel


class FileContentsSectionsItem(UncheckedBaseModel):
    start_page_number: typing_extensions.Annotated[int, FieldMetadata(alias="startPageNumber")] = pydantic.Field(
        alias="startPageNumber"
    )
    """
    The starting page number of this section.
    """

    end_page_number: typing_extensions.Annotated[int, FieldMetadata(alias="endPageNumber")] = pydantic.Field(
        alias="endPageNumber"
    )
    """
    The ending page number of this section.
    """

    markdown: str = pydantic.Field()
    """
    The markdown content of this section.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class ParseConfigAdvancedOptionsReturnOcrParams(typing_extensions.TypedDict):
    """
    Options for returning raw OCR data in the response.
    """

    words: typing_extensions.NotRequired[bool]
    """
    Whether to include word-level OCR data in the response, including word bounding boxes and confidence scores. This meaningfully impacts the response size, consider using the signed url return format.
    """



# This file was auto-generated by Fern from our API Definition.

import enum


class ExtendEnvironment(enum.Enum):
    PRODUCTION = "https://api.extend.ai"
    PRODUCTION_US2 = "https://api.us2.extend.app"
    PRODUCTION_EU1 = "https://api.eu1.extend.ai"



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class ClassificationParams(typing_extensions.TypedDict):
    id: str
    """
    Unique identifier for the classification. We recommend lowercase, underscore-separated format.
    """

    type: str
    """
    Type identifier for the classification.
    """

    description: str
    """
    A detailed description of the classification.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..core.serialization import FieldMetadata


class ParseConfigBlockOptionsTextAgenticParams(typing_extensions.TypedDict):
    """
    Options for agentic text processing using VLM-based review and correction.
    """

    enabled: typing_extensions.NotRequired[bool]
    """
    Whether to enable agentic text processing for OCR corrections.
    """

    custom_instructions: typing_extensions.NotRequired[
        typing_extensions.Annotated[str, FieldMetadata(alias="customInstructions")]
    ]
    """
    Custom instructions to guide the agentic text processing.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

from .classify_step_run import ClassifyStepRunParams
from .conditional_extract_step_run import ConditionalExtractStepRunParams
from .external_data_validation_step_run import ExternalDataValidationStepRunParams
from .extract_step_run import ExtractStepRunParams
from .merge_extract_step_run import MergeExtractStepRunParams
from .parse_step_run import ParseStepRunParams
from .rule_validation_step_run import RuleValidationStepRunParams
from .split_step_run import SplitStepRunParams

StepRunParams = typing.Union[
    ParseStepRunParams,
    ExtractStepRunParams,
    ClassifyStepRunParams,
    SplitStepRunParams,
    MergeExtractStepRunParams,
    ConditionalExtractStepRunParams,
    RuleValidationStepRunParams,
    ExternalDataValidationStepRunParams,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class SplitBaseProcessor(enum.StrEnum):
    """
    The base processor to use. For splitters, this can be either `"splitting_performance"` or `"splitting_light"`. Defaults to `"splitting_performance"` if not provided. See [Splitting Changelog](https://docs.extend.ai/2026-02-09/changelog/splitting/splitting-performance) for more details.
    """

    SPLITTING_PERFORMANCE = "splitting_performance"
    SPLITTING_LIGHT = "splitting_light"
    _UNKNOWN = "__SPLITBASEPROCESSOR_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "SplitBaseProcessor":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        splitting_performance: typing.Callable[[], T_Result],
        splitting_light: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is SplitBaseProcessor.SPLITTING_PERFORMANCE:
            return splitting_performance()
        if self is SplitBaseProcessor.SPLITTING_LIGHT:
            return splitting_light()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class FileType(enum.StrEnum):
    """
    The type of the file based on the file extension or content type.
    """

    PDF = "PDF"
    CSV = "CSV"
    IMG = "IMG"
    TXT = "TXT"
    DOCX = "DOCX"
    EXCEL = "EXCEL"
    XML = "XML"
    HTML = "HTML"
    _UNKNOWN = "__FILETYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "FileType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        pdf: typing.Callable[[], T_Result],
        csv: typing.Callable[[], T_Result],
        img: typing.Callable[[], T_Result],
        txt: typing.Callable[[], T_Result],
        docx: typing.Callable[[], T_Result],
        excel: typing.Callable[[], T_Result],
        xml: typing.Callable[[], T_Result],
        html: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is FileType.PDF:
            return pdf()
        if self is FileType.CSV:
            return csv()
        if self is FileType.IMG:
            return img()
        if self is FileType.TXT:
            return txt()
        if self is FileType.DOCX:
            return docx()
        if self is FileType.EXCEL:
            return excel()
        if self is FileType.XML:
            return xml()
        if self is FileType.HTML:
            return html()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ...types.run_metadata import RunMetadata
from ...types.run_secrets import RunSecrets
from .workflow_runs_create_batch_request_inputs_item_file import WorkflowRunsCreateBatchRequestInputsItemFileParams


class WorkflowRunsCreateBatchRequestInputsItemParams(typing_extensions.TypedDict):
    file: WorkflowRunsCreateBatchRequestInputsItemFileParams
    """
    The file to be processed. Files can be provided as a URL, an Extend file ID, or raw text.
    """

    metadata: typing_extensions.NotRequired[RunMetadata]
    secrets: typing_extensions.NotRequired[RunSecrets]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..core.serialization import FieldMetadata


class ProvidedSplitOutputSplitsItemParams(typing_extensions.TypedDict):
    classification_id: typing_extensions.Annotated[str, FieldMetadata(alias="classificationId")]
    """
    ID of the classification type (defined in the splitter config)
    """

    type: str
    """
    The type of the split document (defined in the splitter config), corresponds to the classificationId
    """

    start_page: typing_extensions.Annotated[int, FieldMetadata(alias="startPage")]
    """
    The start page of the split document
    """

    end_page: typing_extensions.Annotated[int, FieldMetadata(alias="endPage")]
    """
    The end page of the split document
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..core.serialization import FieldMetadata
from ..types.created_at import CreatedAt
from ..types.file_type import FileType
from ..types.updated_at import UpdatedAt
from .file_contents import FileContentsParams
from .file_metadata import FileMetadataParams


class FileParams(typing_extensions.TypedDict):
    object: typing.Literal["file"]
    """
    The type of object. Will always be `"file"`.
    """

    id: str
    """
    ID for the file.
    
    Example: `"file_xK9mLPqRtN3vS8wF5hB2cQ"`
    """

    name: str
    """
    The name of the file
    
    Example: `"Invoices.pdf"`
    """

    type: typing.Optional[FileType]
    """
    The type of the file.
    
    **Availability:** Present when the file type could be determined.
    """

    presigned_url: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="presignedUrl")]
    """
    A presigned URL to download the file. Expires after 15 minutes.
    
    **Availability:** Present on `GET /files/{id}`. Not present on `POST /files/upload` or when the file is embedded in other resources (e.g., in run responses).
    """

    parent_file_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="parentFileId")]
    """
    ID of the parent file.
    
    **Availability:** Present for files created via a Splitter in a workflow.
    """

    contents: typing.Optional[FileContentsParams]
    """
    **Deprecated:** Use the `POST /parse_runs` endpoint instead to parse and retrieve file contents. The parse runs endpoint provides more control over parsing configuration and better performance.
    
    The parsed content of the file. This field will only contain data after the file has been parsed via a parse run, extract run, classify run, split run, edit run, or workflow run.
    
    **Availability:** Only present and populated on `GET /files/{id}` when the file has been previously parsed and the corresponding query parameters are set to true. Will be `null` on `POST /files/upload` and for files that haven't been parsed. The structure varies based on file type.
    """

    metadata: FileMetadataParams
    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")]
    updated_at: typing_extensions.Annotated[UpdatedAt, FieldMetadata(alias="updatedAt")]



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class WorkflowStepBaseParams(typing_extensions.TypedDict):
    """
    Base properties shared by all workflow step types.
    """

    object: typing.Literal["workflow_step"]
    """
    The type of object. In this case, it will always be `"workflow_step"`.
    """

    id: str
    """
    The ID of the workflow step.
    
    Example: `"step_xKm9pNv3qWsY_jL2tR5Dh"`
    """

    name: str
    """
    The name of the workflow step.
    
    Example: `"Validate Invoice Total"`
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..core.serialization import FieldMetadata
from ..types.created_at import CreatedAt
from ..types.processor_run_status import ProcessorRunStatus
from ..types.run_metadata import RunMetadata
from ..types.updated_at import UpdatedAt
from .extractor_summary import ExtractorSummaryParams
from .extractor_version_summary import ExtractorVersionSummaryParams
from .file_summary import FileSummaryParams
from .run_usage import RunUsageParams


class ExtractRunSummaryParams(typing_extensions.TypedDict):
    """
    Summary representation of an extract run.
    """

    object: typing.Literal["extract_run"]
    """
    The type of object. Will always be `"extract_run"`.
    """

    id: str
    """
    The unique identifier for this extract run.
    """

    extractor: typing.Optional[ExtractorSummaryParams]
    """
    The extractor that was used for this run.
    
    **Availability:** Present when an extractor reference was provided. Not present when using inline `config`.
    """

    extractor_version: typing_extensions.Annotated[
        typing.Optional[ExtractorVersionSummaryParams], FieldMetadata(alias="extractorVersion")
    ]
    """
    The version of the extractor that was used for this run.
    
    **Availability:** Present when an extractor reference was provided. Not present when using inline `config`.
    """

    status: ProcessorRunStatus
    failure_reason: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="failureReason")]
    """
    The reason for failure.
    
    **Availability:** Present when `status` is `"FAILED"`.
    """

    failure_message: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="failureMessage")]
    """
    A detailed message about the failure.
    
    **Availability:** Present when `status` is `"FAILED"`.
    """

    metadata: typing.Optional[RunMetadata]
    """
    Any metadata that was provided when creating the extract run.
    
    **Availability:** Present when metadata was provided during creation.
    """

    reviewed: bool
    """
    Indicates whether the run has been reviewed by a human.
    """

    edited: bool
    """
    Indicates whether the run results have been edited during review.
    """

    file: FileSummaryParams
    """
    The file that was processed.
    """

    parse_run_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="parseRunId")]
    """
    The ID of the parse run that was used for this extract run.
    
    **Availability:** Present when a parse run was created.
    """

    dashboard_url: typing_extensions.Annotated[str, FieldMetadata(alias="dashboardUrl")]
    """
    The URL to view the extract run in the Extend dashboard.
    """

    usage: typing.Optional[RunUsageParams]
    """
    Usage credits consumed by this run.
    
    **Availability:** Present when `status` is `"PROCESSED"`.
    """

    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")]
    updated_at: typing_extensions.Annotated[UpdatedAt, FieldMetadata(alias="updatedAt")]



"""
Extended Extend client with webhook utilities and polling methods.

This file is protected by .fernignore and will not be overwritten during regeneration.

Example:
    from extend_ai import Extend

    client = Extend(token="your-api-key")

    # Use create_and_poll for convenient polling
    extract_result = client.extract_runs.create_and_poll(
        file={"url": "https://example.com/doc.pdf"},
        extractor={"id": "extractor_123"},
    )

    classify_result = client.classify_runs.create_and_poll(
        file={"url": "https://example.com/doc.pdf"},
        classifier={"id": "classifier_123"},
    )

    workflow_result = client.workflow_runs.create_and_poll(
        file={"url": "https://example.com/doc.pdf"},
        workflow={"id": "workflow_123"},
    )

    # Verify webhooks
    event = client.webhooks.verify_and_parse(body, headers, secret)
"""

import typing

import httpx

from ..client import AsyncExtend as GeneratedAsyncExtend
from ..client import Extend as GeneratedExtend
from ..environment import ExtendEnvironment

# Import all client types for proper type annotations
from ..files.client import FilesClient, AsyncFilesClient
from ..extractors.client import ExtractorsClient, AsyncExtractorsClient
from ..extractor_versions.client import ExtractorVersionsClient, AsyncExtractorVersionsClient
from ..classifiers.client import ClassifiersClient, AsyncClassifiersClient
from ..classifier_versions.client import ClassifierVersionsClient, AsyncClassifierVersionsClient
from ..splitters.client import SplittersClient, AsyncSplittersClient
from ..splitter_versions.client import SplitterVersionsClient, AsyncSplitterVersionsClient
from ..workflows.client import WorkflowsClient, AsyncWorkflowsClient
from ..evaluation_sets.client import EvaluationSetsClient, AsyncEvaluationSetsClient
from ..evaluation_set_items.client import EvaluationSetItemsClient, AsyncEvaluationSetItemsClient
from ..evaluation_set_runs.client import EvaluationSetRunsClient, AsyncEvaluationSetRunsClient
from ..processor.client import ProcessorClient, AsyncProcessorClient
from ..processor_run.client import ProcessorRunClient, AsyncProcessorRunClient
from ..processor_version.client import ProcessorVersionClient, AsyncProcessorVersionClient
from ..batch_processor_run.client import BatchProcessorRunClient, AsyncBatchProcessorRunClient

from .resources import (
    AsyncClassifyRunsClient,
    AsyncEditRunsClient,
    AsyncExtractRunsClient,
    AsyncParseRunsClient,
    AsyncSplitRunsClient,
    AsyncWorkflowRunsClient,
    ClassifyRunsClient,
    EditRunsClient,
    ExtractRunsClient,
    ParseRunsClient,
    SplitRunsClient,
    WorkflowRunsClient,
)
from .webhooks import Webhooks


class Extend(GeneratedExtend):
    """
    The Extend API client with webhook utilities and polling methods.

    This client extends the generated Extend client with:
    - `create_and_poll()` methods on run resources for convenient polling
    - `webhooks` property for webhook signature verification

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : ExtendEnvironment
        The environment to use for requests from the client.
        Defaults to ExtendEnvironment.PRODUCTION

    token : typing.Union[str, typing.Callable[[], str]]
        Your Extend API token.

    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests.
        Default: 300 seconds.

    follow_redirects : typing.Optional[bool]
        Whether the client follows redirects.

    httpx_client : typing.Optional[httpx.Client]
        Custom httpx client to use for making requests.

    extend_api_version : typing.Optional[str]
        API version to use.

    Examples
    --------
    from extend_ai import Extend

    client = Extend(token="YOUR_TOKEN")

    # Create and poll for extract run
    result = client.extract_runs.create_and_poll(
        file={"url": "https://example.com/doc.pdf"},
        extractor={"id": "extractor_123"}
    )

    # Verify webhook signature
    event = client.webhooks.verify_and_parse(body, headers, secret)
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: ExtendEnvironment = ExtendEnvironment.PRODUCTION,
        token: typing.Union[str, typing.Callable[[], str]],
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
        extend_api_version: typing.Optional[str] = None,
    ):
        super().__init__(
            base_url=base_url,
            environment=environment,
            token=token,
            headers=headers,
            timeout=timeout,
            follow_redirects=follow_redirects,
            httpx_client=httpx_client,
            extend_api_version=extend_api_version,
        )

        # Webhook utilities
        self._webhooks = Webhooks()

        # Client instances (lazy initialization)
        self._extract_runs_client: typing.Optional[ExtractRunsClient] = None
        self._classify_runs_client: typing.Optional[ClassifyRunsClient] = None
        self._split_runs_client: typing.Optional[SplitRunsClient] = None
        self._workflow_runs_client: typing.Optional[WorkflowRunsClient] = None
        self._edit_runs_client: typing.Optional[EditRunsClient] = None
        self._parse_runs_client: typing.Optional[ParseRunsClient] = None

    @property
    def webhooks(self) -> Webhooks:
        """Webhook utilities for signature verification and event parsing."""
        return self._webhooks

    # Run resources with create_and_poll support
    @property
    def extract_runs(self) -> ExtractRunsClient:
        """ExtractRuns client with create_and_poll method."""
        if self._extract_runs_client is None:
            self._extract_runs_client = ExtractRunsClient(client_wrapper=self._client_wrapper)
        return self._extract_runs_client

    @property
    def classify_runs(self) -> ClassifyRunsClient:
        """ClassifyRuns client with create_and_poll method."""
        if self._classify_runs_client is None:
            self._classify_runs_client = ClassifyRunsClient(client_wrapper=self._client_wrapper)
        return self._classify_runs_client

    @property
    def split_runs(self) -> SplitRunsClient:
        """SplitRuns client with create_and_poll method."""
        if self._split_runs_client is None:
            self._split_runs_client = SplitRunsClient(client_wrapper=self._client_wrapper)
        return self._split_runs_client

    @property
    def workflow_runs(self) -> WorkflowRunsClient:
        """WorkflowRuns client with create_and_poll method."""
        if self._workflow_runs_client is None:
            self._workflow_runs_client = WorkflowRunsClient(client_wrapper=self._client_wrapper)
        return self._workflow_runs_client

    @property
    def edit_runs(self) -> EditRunsClient:
        """EditRuns client with create_and_poll method."""
        if self._edit_runs_client is None:
            self._edit_runs_client = EditRunsClient(client_wrapper=self._client_wrapper)
        return self._edit_runs_client

    @property
    def parse_runs(self) -> ParseRunsClient:
        """ParseRuns client with create_and_poll method."""
        if self._parse_runs_client is None:
            self._parse_runs_client = ParseRunsClient(client_wrapper=self._client_wrapper)
        return self._parse_runs_client

    # Type-annotated properties for IDE support (delegate to parent)
    @property
    def files(self) -> FilesClient:
        """Files client."""
        return super().files  # type: ignore[return-value]

    @property
    def extractors(self) -> ExtractorsClient:
        """Extractors client."""
        return super().extractors  # type: ignore[return-value]

    @property
    def extractor_versions(self) -> ExtractorVersionsClient:
        """Extractor versions client."""
        return super().extractor_versions  # type: ignore[return-value]

    @property
    def classifiers(self) -> ClassifiersClient:
        """Classifiers client."""
        return super().classifiers  # type: ignore[return-value]

    @property
    def classifier_versions(self) -> ClassifierVersionsClient:
        """Classifier versions client."""
        return super().classifier_versions  # type: ignore[return-value]

    @property
    def splitters(self) -> SplittersClient:
        """Splitters client."""
        return super().splitters  # type: ignore[return-value]

    @property
    def splitter_versions(self) -> SplitterVersionsClient:
        """Splitter versions client."""
        return super().splitter_versions  # type: ignore[return-value]

    @property
    def workflows(self) -> WorkflowsClient:
        """Workflows client."""
        return super().workflows  # type: ignore[return-value]

    @property
    def evaluation_sets(self) -> EvaluationSetsClient:
        """Evaluation sets client."""
        return super().evaluation_sets  # type: ignore[return-value]

    @property
    def evaluation_set_items(self) -> EvaluationSetItemsClient:
        """Evaluation set items client."""
        return super().evaluation_set_items  # type: ignore[return-value]

    @property
    def evaluation_set_runs(self) -> EvaluationSetRunsClient:
        """Evaluation set runs client."""
        return super().evaluation_set_runs  # type: ignore[return-value]

    @property
    def processor(self) -> ProcessorClient:
        """Processor client (legacy)."""
        return super().processor  # type: ignore[return-value]

    @property
    def processor_run(self) -> ProcessorRunClient:
        """Processor run client (legacy)."""
        return super().processor_run  # type: ignore[return-value]

    @property
    def processor_version(self) -> ProcessorVersionClient:
        """Processor version client (legacy)."""
        return super().processor_version  # type: ignore[return-value]

    @property
    def batch_processor_run(self) -> BatchProcessorRunClient:
        """Batch processor run client."""
        return super().batch_processor_run  # type: ignore[return-value]


class AsyncExtend(GeneratedAsyncExtend):
    """
    The async Extend API client with webhook utilities and polling methods.

    This client extends the generated AsyncExtend client with:
    - `create_and_poll()` methods on run resources for convenient polling
    - `webhooks` property for webhook signature verification

    Examples
    --------
    import asyncio
    from extend_ai import AsyncExtend

    client = AsyncExtend(token="YOUR_TOKEN")

    async def main():
        result = await client.extract_runs.create_and_poll(
            file={"url": "https://example.com/doc.pdf"},
            extractor={"id": "extractor_123"}
        )

    asyncio.run(main())
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: ExtendEnvironment = ExtendEnvironment.PRODUCTION,
        token: typing.Union[str, typing.Callable[[], str]],
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
        extend_api_version: typing.Optional[str] = None,
    ):
        super().__init__(
            base_url=base_url,
            environment=environment,
            token=token,
            headers=headers,
            timeout=timeout,
            follow_redirects=follow_redirects,
            httpx_client=httpx_client,
            extend_api_version=extend_api_version,
        )

        # Webhook utilities
        self._webhooks = Webhooks()

        # Client instances (lazy initialization)
        self._extract_runs_client: typing.Optional[AsyncExtractRunsClient] = None
        self._classify_runs_client: typing.Optional[AsyncClassifyRunsClient] = None
        self._split_runs_client: typing.Optional[AsyncSplitRunsClient] = None
        self._workflow_runs_client: typing.Optional[AsyncWorkflowRunsClient] = None
        self._edit_runs_client: typing.Optional[AsyncEditRunsClient] = None
        self._parse_runs_client: typing.Optional[AsyncParseRunsClient] = None

    @property
    def webhooks(self) -> Webhooks:
        """Webhook utilities for signature verification and event parsing."""
        return self._webhooks

    # Run resources with create_and_poll support
    @property
    def extract_runs(self) -> AsyncExtractRunsClient:
        """ExtractRuns client with create_and_poll method."""
        if self._extract_runs_client is None:
            self._extract_runs_client = AsyncExtractRunsClient(client_wrapper=self._client_wrapper)
        return self._extract_runs_client

    @property
    def classify_runs(self) -> AsyncClassifyRunsClient:
        """ClassifyRuns client with create_and_poll method."""
        if self._classify_runs_client is None:
            self._classify_runs_client = AsyncClassifyRunsClient(client_wrapper=self._client_wrapper)
        return self._classify_runs_client

    @property
    def split_runs(self) -> AsyncSplitRunsClient:
        """SplitRuns client with create_and_poll method."""
        if self._split_runs_client is None:
            self._split_runs_client = AsyncSplitRunsClient(client_wrapper=self._client_wrapper)
        return self._split_runs_client

    @property
    def workflow_runs(self) -> AsyncWorkflowRunsClient:
        """WorkflowRuns client with create_and_poll method."""
        if self._workflow_runs_client is None:
            self._workflow_runs_client = AsyncWorkflowRunsClient(client_wrapper=self._client_wrapper)
        return self._workflow_runs_client

    @property
    def edit_runs(self) -> AsyncEditRunsClient:
        """EditRuns client with create_and_poll method."""
        if self._edit_runs_client is None:
            self._edit_runs_client = AsyncEditRunsClient(client_wrapper=self._client_wrapper)
        return self._edit_runs_client

    @property
    def parse_runs(self) -> AsyncParseRunsClient:
        """ParseRuns client with create_and_poll method."""
        if self._parse_runs_client is None:
            self._parse_runs_client = AsyncParseRunsClient(client_wrapper=self._client_wrapper)
        return self._parse_runs_client

    # Type-annotated properties for IDE support (delegate to parent)
    @property
    def files(self) -> AsyncFilesClient:
        """Files client."""
        return super().files  # type: ignore[return-value]

    @property
    def extractors(self) -> AsyncExtractorsClient:
        """Extractors client."""
        return super().extractors  # type: ignore[return-value]

    @property
    def extractor_versions(self) -> AsyncExtractorVersionsClient:
        """Extractor versions client."""
        return super().extractor_versions  # type: ignore[return-value]

    @property
    def classifiers(self) -> AsyncClassifiersClient:
        """Classifiers client."""
        return super().classifiers  # type: ignore[return-value]

    @property
    def classifier_versions(self) -> AsyncClassifierVersionsClient:
        """Classifier versions client."""
        return super().classifier_versions  # type: ignore[return-value]

    @property
    def splitters(self) -> AsyncSplittersClient:
        """Splitters client."""
        return super().splitters  # type: ignore[return-value]

    @property
    def splitter_versions(self) -> AsyncSplitterVersionsClient:
        """Splitter versions client."""
        return super().splitter_versions  # type: ignore[return-value]

    @property
    def workflows(self) -> AsyncWorkflowsClient:
        """Workflows client."""
        return super().workflows  # type: ignore[return-value]

    @property
    def evaluation_sets(self) -> AsyncEvaluationSetsClient:
        """Evaluation sets client."""
        return super().evaluation_sets  # type: ignore[return-value]

    @property
    def evaluation_set_items(self) -> AsyncEvaluationSetItemsClient:
        """Evaluation set items client."""
        return super().evaluation_set_items  # type: ignore[return-value]

    @property
    def evaluation_set_runs(self) -> AsyncEvaluationSetRunsClient:
        """Evaluation set runs client."""
        return super().evaluation_set_runs  # type: ignore[return-value]

    @property
    def processor(self) -> AsyncProcessorClient:
        """Processor client (legacy)."""
        return super().processor  # type: ignore[return-value]

    @property
    def processor_run(self) -> AsyncProcessorRunClient:
        """Processor run client (legacy)."""
        return super().processor_run  # type: ignore[return-value]

    @property
    def processor_version(self) -> AsyncProcessorVersionClient:
        """Processor version client (legacy)."""
        return super().processor_version  # type: ignore[return-value]

    @property
    def batch_processor_run(self) -> AsyncBatchProcessorRunClient:
        """Batch processor run client."""
        return super().batch_processor_run  # type: ignore[return-value]



# This file was auto-generated by Fern from our API Definition.

import typing

from .extract_output_metadata_value import ExtractOutputMetadataValue

ExtractOutputMetadata = typing.Dict[str, ExtractOutputMetadataValue]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel


class FileContentsSheetsItem(UncheckedBaseModel):
    sheet_name: typing_extensions.Annotated[str, FieldMetadata(alias="sheetName")] = pydantic.Field(alias="sheetName")
    """
    The name of the sheet.
    """

    raw_text: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="rawText")] = pydantic.Field(
        alias="rawText", default=None
    )
    """
    The raw text content of the sheet in CSV format.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .created_at import CreatedAt
from .extract_config import ExtractConfig


class ExtractorVersion(UncheckedBaseModel):
    object: typing.Literal["extractor_version"] = pydantic.Field(default="extractor_version")
    """
    The type of object. Will always be `"extractor_version"`.
    """

    id: str = pydantic.Field()
    """
    The unique identifier for this version of the extractor.
    
    Example: `"exv_xK9mLPqRtN3vS8wF5hB2cQ"`
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    A description of this version of the extractor.
    
    Example: `"Updated extraction fields for new invoice format"`
    """

    version: str = pydantic.Field()
    """
    The version number or identifier for this specific version of the extractor. The draft version will have version="draft".
    
    Examples: `"1.0"`, `"2.1"`, `"draft"`
    """

    config: ExtractConfig = pydantic.Field()
    """
    The configuration settings for this version of the extractor. This is a union of two possible shapes:
    
    - **[JSON Schema config](https://docs.extend.ai/2026-02-09/product/extraction/schema):** The current config format. All extractors created through this API version use this shape.
    - **[Legacy config](https://docs.extend.ai/2025-04-21/product/legacy/legacy-schema):** A fields-array config from a previous API version. This shape is only returned for extractors that were originally configured with the legacy format. This API version does not support creating extractors with legacy configs.
    """

    extractor_id: typing_extensions.Annotated[str, FieldMetadata(alias="extractorId")] = pydantic.Field(
        alias="extractorId"
    )
    """
    The ID of the extractor that this version belongs to.
    
    Example: `"ex_Xj8mK2pL9nR4vT7qY5wZ"`
    """

    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")] = pydantic.Field(
        alias="createdAt"
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


update_forward_refs(ExtractorVersion)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions
from ..core.serialization import FieldMetadata
from ..types.created_at import CreatedAt
from ..types.updated_at import UpdatedAt


class BatchRunEntity_ExtractorParams(typing_extensions.TypedDict):
    object: typing.Literal["extractor"]
    id: str
    name: str
    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")]
    updated_at: typing_extensions.Annotated[UpdatedAt, FieldMetadata(alias="updatedAt")]


class BatchRunEntity_ClassifierParams(typing_extensions.TypedDict):
    object: typing.Literal["classifier"]
    id: str
    name: str
    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")]
    updated_at: typing_extensions.Annotated[UpdatedAt, FieldMetadata(alias="updatedAt")]


class BatchRunEntity_SplitterParams(typing_extensions.TypedDict):
    object: typing.Literal["splitter"]
    id: str
    name: str
    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")]
    updated_at: typing_extensions.Annotated[UpdatedAt, FieldMetadata(alias="updatedAt")]


BatchRunEntityParams = typing.Union[
    BatchRunEntity_ExtractorParams, BatchRunEntity_ClassifierParams, BatchRunEntity_SplitterParams
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .block_metadata_page import BlockMetadataPageParams


class BlockMetadataParams(typing_extensions.TypedDict):
    """
    Metadata about the block.
    """

    page: typing_extensions.NotRequired[BlockMetadataPageParams]
    """
    Information about the page this block appears on.
    """



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        WorkflowRunsCreateBatchRequestInputsItem,
        WorkflowRunsCreateBatchRequestInputsItemFile,
        WorkflowRunsCreateBatchResponse,
        WorkflowRunsCreateRequestFile,
        WorkflowRunsCreateRequestOutputsItem,
        WorkflowRunsDeleteResponse,
        WorkflowRunsListResponse,
    )
    from .requests import (
        WorkflowRunsCreateBatchRequestInputsItemFileParams,
        WorkflowRunsCreateBatchRequestInputsItemParams,
        WorkflowRunsCreateBatchResponseParams,
        WorkflowRunsCreateRequestFileParams,
        WorkflowRunsCreateRequestOutputsItemParams,
        WorkflowRunsDeleteResponseParams,
        WorkflowRunsListResponseParams,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "WorkflowRunsCreateBatchRequestInputsItem": ".types",
    "WorkflowRunsCreateBatchRequestInputsItemFile": ".types",
    "WorkflowRunsCreateBatchRequestInputsItemFileParams": ".requests",
    "WorkflowRunsCreateBatchRequestInputsItemParams": ".requests",
    "WorkflowRunsCreateBatchResponse": ".types",
    "WorkflowRunsCreateBatchResponseParams": ".requests",
    "WorkflowRunsCreateRequestFile": ".types",
    "WorkflowRunsCreateRequestFileParams": ".requests",
    "WorkflowRunsCreateRequestOutputsItem": ".types",
    "WorkflowRunsCreateRequestOutputsItemParams": ".requests",
    "WorkflowRunsDeleteResponse": ".types",
    "WorkflowRunsDeleteResponseParams": ".requests",
    "WorkflowRunsListResponse": ".types",
    "WorkflowRunsListResponseParams": ".requests",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "WorkflowRunsCreateBatchRequestInputsItem",
    "WorkflowRunsCreateBatchRequestInputsItemFile",
    "WorkflowRunsCreateBatchRequestInputsItemFileParams",
    "WorkflowRunsCreateBatchRequestInputsItemParams",
    "WorkflowRunsCreateBatchResponse",
    "WorkflowRunsCreateBatchResponseParams",
    "WorkflowRunsCreateRequestFile",
    "WorkflowRunsCreateRequestFileParams",
    "WorkflowRunsCreateRequestOutputsItem",
    "WorkflowRunsCreateRequestOutputsItemParams",
    "WorkflowRunsDeleteResponse",
    "WorkflowRunsDeleteResponseParams",
    "WorkflowRunsListResponse",
    "WorkflowRunsListResponseParams",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from ...core.unchecked_base_model import UncheckedBaseModel
from ...types.processor_version_string import ProcessorVersionString
from ...types.split_override_config import SplitOverrideConfig


class SplitRunsCreateRequestSplitter(UncheckedBaseModel):
    """
    Reference to an existing splitter. One of `splitter` or `config` must be provided.
    """

    id: str = pydantic.Field()
    """
    The ID of the splitter to use.
    """

    version: typing.Optional[ProcessorVersionString] = None
    override_config: typing_extensions.Annotated[
        typing.Optional[SplitOverrideConfig], FieldMetadata(alias="overrideConfig")
    ] = pydantic.Field(alias="overrideConfig", default=None)
    """
    Optional partial configuration override. Only the fields you provide will override the splitter's saved configuration. For example, you can pass only `splitRules` without providing `splitClassifications`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



"""
Extended WorkflowRuns client with polling utilities.

Example:
    from extend_ai import Extend

    client = Extend(token="...")

    # Create and poll until completion
    result = client.workflow_runs.create_and_poll(
        file={"id": "file_xxx"},
        workflow={"id": "workflow_abc123"},
    )

    if result.status == "PROCESSED":
        print(result.step_runs)
"""

from typing import Any, Dict, Optional, Sequence

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...requests.workflow_reference import WorkflowReferenceParams
from ...types.run_metadata import RunMetadata
from ...types.run_priority import RunPriority
from ...types.run_secrets import RunSecrets
from ...types.workflow_run import WorkflowRun
from ...workflow_runs.client import AsyncWorkflowRunsClient as GeneratedAsyncWorkflowRunsClient
from ...workflow_runs.client import WorkflowRunsClient as GeneratedWorkflowRunsClient
from ...workflow_runs.requests.workflow_runs_create_request_file import WorkflowRunsCreateRequestFileParams
from ...workflow_runs.requests.workflow_runs_create_request_outputs_item import WorkflowRunsCreateRequestOutputsItemParams
from ..polling import PollingOptions, poll_until_done, poll_until_done_async

# Re-export for convenience
from ..polling import PollingTimeoutError

__all__ = ["WorkflowRunsClient", "AsyncWorkflowRunsClient", "PollingTimeoutError"]


def _is_terminal_status(status: str) -> bool:
    """
    Check if a WorkflowRunStatus is terminal (no longer processing).
    We check for non-terminal states rather than terminal states so that
    if new terminal states are added, polling will still complete.

    Non-terminal states: PENDING, PROCESSING, CANCELLING
    Terminal states: PROCESSED, FAILED, CANCELLED, NEEDS_REVIEW, REJECTED
    """
    return status not in ("PROCESSING", "PENDING", "CANCELLING")


class WorkflowRunsClient(GeneratedWorkflowRunsClient):
    """
    Extended WorkflowRuns client with create_and_poll method.

    Inherits all methods from WorkflowRunsClient and adds create_and_poll
    for convenient polling until completion.
    """

    def __init__(self, *, client_wrapper: SyncClientWrapper):
        super().__init__(client_wrapper=client_wrapper)

    def create_and_poll(
        self,
        *,
        workflow: WorkflowReferenceParams,
        file: WorkflowRunsCreateRequestFileParams,
        outputs: Optional[Sequence[WorkflowRunsCreateRequestOutputsItemParams]] = None,
        priority: Optional[RunPriority] = None,
        metadata: Optional[RunMetadata] = None,
        secrets: Optional[RunSecrets] = None,
        polling_options: Optional[PollingOptions] = None,
    ) -> WorkflowRun:
        """
        Creates a workflow run and polls until it reaches a terminal state.

        This is a convenience method that combines create() and polling via
        retrieve() with exponential backoff and jitter.

        Terminal states: PROCESSED, FAILED, CANCELLED, NEEDS_REVIEW, REJECTED

        Args:
            workflow: Reference to the workflow to run.
            file: The file to process.
            outputs: Optional list of output configurations.
            priority: Priority of the run.
            metadata: Additional metadata for the run.
            secrets: Secret values for the run.
            polling_options: Options for polling behavior.

        Returns:
            The final workflow run when processing is complete.

        Raises:
            PollingTimeoutError: If max_wait_ms is set and exceeded.

        Example:
            result = client.workflow_runs.create_and_poll(
                file={"id": "file_xxx"},
                workflow={"id": "workflow_abc123"}
            )

            match result.status:
                case "PROCESSED":
                    print("Success:", result.step_runs)
                case "NEEDS_REVIEW":
                    print("Needs review:", result.dashboard_url)
                case "FAILED":
                    print("Failed:", result.failure_message)
        """
        # Build kwargs, only including non-None values to avoid passing null
        kwargs: Dict[str, Any] = {"workflow": workflow, "file": file}
        if outputs is not None:
            kwargs["outputs"] = outputs
        if priority is not None:
            kwargs["priority"] = priority
        if metadata is not None:
            kwargs["metadata"] = metadata
        if secrets is not None:
            kwargs["secrets"] = secrets

        # Create the workflow run
        create_response = self.create(**kwargs)
        run_id = create_response.id

        # Poll until terminal state
        return poll_until_done(
            retrieve=lambda: self.retrieve(run_id),
            is_terminal=lambda response: _is_terminal_status(response.status),
            options=polling_options,
        )


class AsyncWorkflowRunsClient(GeneratedAsyncWorkflowRunsClient):
    """
    Extended AsyncWorkflowRuns client with create_and_poll method.
    """

    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        super().__init__(client_wrapper=client_wrapper)

    async def create_and_poll(
        self,
        *,
        workflow: WorkflowReferenceParams,
        file: WorkflowRunsCreateRequestFileParams,
        outputs: Optional[Sequence[WorkflowRunsCreateRequestOutputsItemParams]] = None,
        priority: Optional[RunPriority] = None,
        metadata: Optional[RunMetadata] = None,
        secrets: Optional[RunSecrets] = None,
        polling_options: Optional[PollingOptions] = None,
    ) -> WorkflowRun:
        """
        Creates a workflow run and polls until it reaches a terminal state (async version).
        """
        # Build kwargs, only including non-None values to avoid passing null
        kwargs: Dict[str, Any] = {"workflow": workflow, "file": file}
        if outputs is not None:
            kwargs["outputs"] = outputs
        if priority is not None:
            kwargs["priority"] = priority
        if metadata is not None:
            kwargs["metadata"] = metadata
        if secrets is not None:
            kwargs["secrets"] = secrets

        # Create the workflow run
        create_response = await self.create(**kwargs)
        run_id = create_response.id

        # Poll until terminal state
        return await poll_until_done_async(
            retrieve=lambda: self.retrieve(run_id),
            is_terminal=lambda response: _is_terminal_status(response.status),
            options=polling_options,
        )



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class ApiVersionEnum(enum.StrEnum):
    TWO_THOUSAND_TWENTY_SIX0209 = "2026-02-09"
    TWO_THOUSAND_TWENTY_FIVE0421 = "2025-04-21"
    TWO_THOUSAND_TWENTY_FOUR1223 = "2024-12-23"
    TWO_THOUSAND_TWENTY_FOUR1114 = "2024-11-14"
    TWO_THOUSAND_TWENTY_FOUR0730 = "2024-07-30"
    TWO_THOUSAND_TWENTY_FOUR0201 = "2024-02-01"
    _UNKNOWN = "__APIVERSIONENUM_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ApiVersionEnum":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        two_thousand_twenty_six0209: typing.Callable[[], T_Result],
        two_thousand_twenty_five0421: typing.Callable[[], T_Result],
        two_thousand_twenty_four1223: typing.Callable[[], T_Result],
        two_thousand_twenty_four1114: typing.Callable[[], T_Result],
        two_thousand_twenty_four0730: typing.Callable[[], T_Result],
        two_thousand_twenty_four0201: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ApiVersionEnum.TWO_THOUSAND_TWENTY_SIX0209:
            return two_thousand_twenty_six0209()
        if self is ApiVersionEnum.TWO_THOUSAND_TWENTY_FIVE0421:
            return two_thousand_twenty_five0421()
        if self is ApiVersionEnum.TWO_THOUSAND_TWENTY_FOUR1223:
            return two_thousand_twenty_four1223()
        if self is ApiVersionEnum.TWO_THOUSAND_TWENTY_FOUR1114:
            return two_thousand_twenty_four1114()
        if self is ApiVersionEnum.TWO_THOUSAND_TWENTY_FOUR0730:
            return two_thousand_twenty_four0730()
        if self is ApiVersionEnum.TWO_THOUSAND_TWENTY_FOUR0201:
            return two_thousand_twenty_four0201()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..core.serialization import FieldMetadata
from ..types.classify_advanced_options_context import ClassifyAdvancedOptionsContext
from .page_ranges import PageRangesParams


class ClassifyAdvancedOptionsParams(typing_extensions.TypedDict):
    context: typing_extensions.NotRequired[ClassifyAdvancedOptionsContext]
    """
    The context to use for classification.
    """

    advanced_multimodal_enabled: typing_extensions.NotRequired[
        typing_extensions.Annotated[bool, FieldMetadata(alias="advancedMultimodalEnabled")]
    ]
    """
    Enable advanced multimodal processing for better handling of visual elements during classification.
    """

    page_ranges: typing_extensions.NotRequired[
        typing_extensions.Annotated[PageRangesParams, FieldMetadata(alias="pageRanges")]
    ]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class ParseConfigBlockOptionsTablesTargetFormat(enum.StrEnum):
    """
    The target format for the table blocks. Supported values:
    * `markdown`: Convert table to Markdown format
    * `html`: Convert table to HTML format
    """

    MARKDOWN = "markdown"
    HTML = "html"
    _UNKNOWN = "__PARSECONFIGBLOCKOPTIONSTABLESTARGETFORMAT_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ParseConfigBlockOptionsTablesTargetFormat":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        markdown: typing.Callable[[], T_Result],
        html: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ParseConfigBlockOptionsTablesTargetFormat.MARKDOWN:
            return markdown()
        if self is ParseConfigBlockOptionsTablesTargetFormat.HTML:
            return html()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        ProcessorRunCancelResponse,
        ProcessorRunCreateRequestConfig,
        ProcessorRunCreateRequestConfig_Classify,
        ProcessorRunCreateRequestConfig_Extract,
        ProcessorRunCreateRequestConfig_Splitter,
        ProcessorRunCreateResponse,
        ProcessorRunDeleteResponse,
        ProcessorRunGetResponse,
        ProcessorRunListRequestSource,
        ProcessorRunListResponse,
    )
    from .requests import (
        ProcessorRunCancelResponseParams,
        ProcessorRunCreateRequestConfigParams,
        ProcessorRunCreateRequestConfig_ClassifyParams,
        ProcessorRunCreateRequestConfig_ExtractParams,
        ProcessorRunCreateRequestConfig_SplitterParams,
        ProcessorRunCreateResponseParams,
        ProcessorRunDeleteResponseParams,
        ProcessorRunGetResponseParams,
        ProcessorRunListResponseParams,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "ProcessorRunCancelResponse": ".types",
    "ProcessorRunCancelResponseParams": ".requests",
    "ProcessorRunCreateRequestConfig": ".types",
    "ProcessorRunCreateRequestConfigParams": ".requests",
    "ProcessorRunCreateRequestConfig_Classify": ".types",
    "ProcessorRunCreateRequestConfig_ClassifyParams": ".requests",
    "ProcessorRunCreateRequestConfig_Extract": ".types",
    "ProcessorRunCreateRequestConfig_ExtractParams": ".requests",
    "ProcessorRunCreateRequestConfig_Splitter": ".types",
    "ProcessorRunCreateRequestConfig_SplitterParams": ".requests",
    "ProcessorRunCreateResponse": ".types",
    "ProcessorRunCreateResponseParams": ".requests",
    "ProcessorRunDeleteResponse": ".types",
    "ProcessorRunDeleteResponseParams": ".requests",
    "ProcessorRunGetResponse": ".types",
    "ProcessorRunGetResponseParams": ".requests",
    "ProcessorRunListRequestSource": ".types",
    "ProcessorRunListResponse": ".types",
    "ProcessorRunListResponseParams": ".requests",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "ProcessorRunCancelResponse",
    "ProcessorRunCancelResponseParams",
    "ProcessorRunCreateRequestConfig",
    "ProcessorRunCreateRequestConfigParams",
    "ProcessorRunCreateRequestConfig_Classify",
    "ProcessorRunCreateRequestConfig_ClassifyParams",
    "ProcessorRunCreateRequestConfig_Extract",
    "ProcessorRunCreateRequestConfig_ExtractParams",
    "ProcessorRunCreateRequestConfig_Splitter",
    "ProcessorRunCreateRequestConfig_SplitterParams",
    "ProcessorRunCreateResponse",
    "ProcessorRunCreateResponseParams",
    "ProcessorRunDeleteResponse",
    "ProcessorRunDeleteResponseParams",
    "ProcessorRunGetResponse",
    "ProcessorRunGetResponseParams",
    "ProcessorRunListRequestSource",
    "ProcessorRunListResponse",
    "ProcessorRunListResponseParams",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel


class LegacyExtendError(UncheckedBaseModel):
    code: str = pydantic.Field()
    """
    Error code identifying the type of error
    """

    message: str = pydantic.Field()
    """
    Human-readable error message
    """

    request_id: typing_extensions.Annotated[str, FieldMetadata(alias="requestId")] = pydantic.Field(alias="requestId")
    """
    Unique request identifier for support purposes
    """

    retryable: bool = pydantic.Field()
    """
    Whether the request can be retried
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel


class EditRootJson(UncheckedBaseModel):
    """
    JSON Schema definition for editing PDF documents. The schema defines the structure and placement of fields to edit.
    """

    type: typing.Literal["object"] = pydantic.Field(default="object")
    """
    Must be "object" for the root schema
    """

    properties: typing.Dict[str, "EditJson"] = pydantic.Field()
    """
    Map of field names to their schema definitions
    """

    required: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of required field names
    """

    additional_properties: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="additionalProperties")
    ] = pydantic.Field(alias="additionalProperties", default=None)
    """
    Whether additional properties are allowed
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .edit_json import EditJson  # noqa: E402, I001
from .edit_object_json import EditObjectJson  # noqa: E402, I001

update_forward_refs(EditRootJson, EditJson=EditJson, EditObjectJson=EditObjectJson)



# This file was auto-generated by Fern from our API Definition.

import typing

from .legacy_classifier_output import LegacyClassifierOutput
from .legacy_extraction_output import LegacyExtractionOutput
from .legacy_splitter_output import LegacySplitterOutput

LegacyProcessorOutput = typing.Union[LegacyExtractionOutput, LegacyClassifierOutput, LegacySplitterOutput]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from ...core.unchecked_base_model import UncheckedBaseModel
from ...types.provided_processor_output import ProvidedProcessorOutput


class WorkflowRunsCreateRequestOutputsItem(UncheckedBaseModel):
    processor_id: typing_extensions.Annotated[str, FieldMetadata(alias="processorId")] = pydantic.Field(
        alias="processorId"
    )
    """
    The ID of the extractor, classifier, or splitter that the output is associated with.
    Example: `"ex_Xj8mK2pL9nR4vT7qY5wZ"`
    """

    output: ProvidedProcessorOutput = pydantic.Field()
    """
    The output to be used for the workflow run. The structure will depend on the processor type. More details can be found on the "output type" page for the corresponding processor, in the guides section.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .parse_config_block_options_figures import ParseConfigBlockOptionsFigures
from .parse_config_block_options_tables import ParseConfigBlockOptionsTables
from .parse_config_block_options_text import ParseConfigBlockOptionsText


class ParseConfigBlockOptions(UncheckedBaseModel):
    """
    Options for controlling how different block types are processed.
    """

    figures: typing.Optional[ParseConfigBlockOptionsFigures] = pydantic.Field(default=None)
    """
    Options for figure blocks.
    """

    tables: typing.Optional[ParseConfigBlockOptionsTables] = pydantic.Field(default=None)
    """
    Options for table blocks.
    """

    text: typing.Optional[ParseConfigBlockOptionsText] = pydantic.Field(default=None)
    """
    Options for text blocks.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..core.serialization import FieldMetadata
from ..types.batch_run_status import BatchRunStatus
from ..types.created_at import CreatedAt
from ..types.updated_at import UpdatedAt
from .evaluation_set_run_entity import EvaluationSetRunEntityParams
from .evaluation_set_run_entity_version import EvaluationSetRunEntityVersionParams
from .evaluation_set_run_metrics import EvaluationSetRunMetricsParams
from .evaluation_set_run_options import EvaluationSetRunOptionsParams


class EvaluationSetRunParams(typing_extensions.TypedDict):
    object: typing.Literal["evaluation_set_run"]
    """
    The type of object. In this case, it will always be `"evaluation_set_run"`.
    """

    id: str
    """
    The unique identifier for this evaluation set run.
    
    Example: `"eval_set_run_Xj8mK2pL9nR4vT7qY5wZ"`
    """

    evaluation_set_id: typing_extensions.Annotated[str, FieldMetadata(alias="evaluationSetId")]
    """
    The ID of the evaluation set that was run.
    
    Example: `"ev_2LcgeY_mp2T5yPaEuq5Lw"`
    """

    entity: EvaluationSetRunEntityParams
    """
    The extractor, classifier, or splitter that was run.
    """

    entity_version: typing_extensions.Annotated[
        EvaluationSetRunEntityVersionParams, FieldMetadata(alias="entityVersion")
    ]
    """
    The version of the extractor, classifier, or splitter that was run.
    """

    metrics: EvaluationSetRunMetricsParams
    status: BatchRunStatus
    options: EvaluationSetRunOptionsParams
    """
    The options for the evaluation set run.
    """

    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")]
    updated_at: typing_extensions.Annotated[UpdatedAt, FieldMetadata(alias="updatedAt")]



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..core.serialization import FieldMetadata


class TableCellDetailsParams(typing_extensions.TypedDict):
    """
    Details for table cell blocks
    """

    type: typing.Literal["table_cell_details"]
    """
    Indicates this is a table cell details object
    """

    row_index: typing_extensions.Annotated[int, FieldMetadata(alias="rowIndex")]
    column_index: typing_extensions.Annotated[int, FieldMetadata(alias="columnIndex")]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..core.serialization import FieldMetadata
from .extract_run import ExtractRunParams


class ExtractResultParams(typing_extensions.TypedDict):
    """
    The result of an extract step.
    """

    extract_run: typing_extensions.Annotated[ExtractRunParams, FieldMetadata(alias="extractRun")]
    """
    The extract run that was executed.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..core.serialization import FieldMetadata
from ..types.created_at import CreatedAt
from ..types.updated_at import UpdatedAt
from .classifier_version import ClassifierVersionParams


class ClassifierParams(typing_extensions.TypedDict):
    object: typing.Literal["classifier"]
    """
    The type of object. Will always be `"classifier"`.
    """

    id: str
    """
    The ID of the classifier.
    
    Example: `"cl_Xj8mK2pL9nR4vT7qY5wZ"`
    """

    name: str
    """
    The name of the classifier.
    
    Example: `"Document Type Classifier"`
    """

    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")]
    updated_at: typing_extensions.Annotated[UpdatedAt, FieldMetadata(alias="updatedAt")]
    draft_version: typing_extensions.Annotated[ClassifierVersionParams, FieldMetadata(alias="draftVersion")]
    """
    The draft version of the classifier. This is the editable version in the Extend dashboard.
    """



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions
from ..core.serialization import FieldMetadata

if typing.TYPE_CHECKING:
    from .edit_json import EditJsonParams


class EditObjectJsonParams(typing_extensions.TypedDict):
    """
    Schema definition for an object field in a PDF edit schema. Used for signature fields and as items schema for arrays/tables.
    """

    type: typing_extensions.NotRequired[typing.Literal["object"]]
    """
    Must be "object" for object schemas
    """

    description: typing_extensions.NotRequired[str]
    """
    Description of the field
    """

    extend_edit_field_type: typing_extensions.NotRequired[
        typing_extensions.Annotated[typing.Literal["signature"], FieldMetadata(alias="extend_edit:field_type")]
    ]
    """
    The PDF field type. For object schemas, must be "signature".
    """

    properties: typing_extensions.NotRequired[typing.Dict[str, "EditJsonParams"]]
    """
    Nested properties. Each property follows the EditJSON structure.
    """

    required: typing_extensions.NotRequired[typing.Sequence[str]]
    """
    List of required property names
    """

    additional_properties: typing_extensions.NotRequired[
        typing_extensions.Annotated[bool, FieldMetadata(alias="additionalProperties")]
    ]
    """
    Whether additional properties are allowed
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .file_summary import FileSummary
from .provided_processor_output import ProvidedProcessorOutput


class EvaluationSetItem(UncheckedBaseModel):
    object: typing.Literal["evaluation_set_item"] = pydantic.Field(default="evaluation_set_item")
    """
    The type of object. In this case, it will always be `"evaluation_set_item"`.
    """

    id: str = pydantic.Field()
    """
    The ID of the evaluation set item.
    
    Example: `"evi_kR9mNP12Qw4yTv8BdR3H"`
    """

    evaluation_set_id: typing_extensions.Annotated[str, FieldMetadata(alias="evaluationSetId")] = pydantic.Field(
        alias="evaluationSetId"
    )
    """
    The ID of the evaluation set that this item belongs to.
    
    Example: `"ev_2LcgeY_mp2T5yPaEuq5Lw"`
    """

    file: FileSummary = pydantic.Field()
    """
    A summary of the file associated with the evaluation set item.
    """

    expected_output: typing_extensions.Annotated[ProvidedProcessorOutput, FieldMetadata(alias="expectedOutput")] = (
        pydantic.Field(alias="expectedOutput")
    )
    """
    The expected output that will be used to evaluate the performance of the extractor, classifier, or splitter associated with the evaluation set. This must conform to the output schema of the entity associated with the evaluation set.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class LegacyProcessorType(enum.StrEnum):
    """
    The type of processor:
    * `"EXTRACT"` - Extracts structured data from documents
    * `"CLASSIFY"` - Classifies documents into categories
    * `"SPLITTER"` - Splits documents into multiple parts
    """

    EXTRACT = "EXTRACT"
    CLASSIFY = "CLASSIFY"
    SPLITTER = "SPLITTER"
    _UNKNOWN = "__LEGACYPROCESSORTYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "LegacyProcessorType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        extract: typing.Callable[[], T_Result],
        classify: typing.Callable[[], T_Result],
        splitter: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is LegacyProcessorType.EXTRACT:
            return extract()
        if self is LegacyProcessorType.CLASSIFY:
            return classify()
        if self is LegacyProcessorType.SPLITTER:
            return splitter()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..core.serialization import FieldMetadata
from ..types.split_base_processor import SplitBaseProcessor
from .classifications import ClassificationsParams
from .parse_config import ParseConfigParams
from .split_advanced_options import SplitAdvancedOptionsParams


class SplitConfigParams(typing_extensions.TypedDict):
    base_processor: typing_extensions.NotRequired[
        typing_extensions.Annotated[SplitBaseProcessor, FieldMetadata(alias="baseProcessor")]
    ]
    base_version: typing_extensions.NotRequired[typing_extensions.Annotated[str, FieldMetadata(alias="baseVersion")]]
    """
    The version of the `"splitting_performance"` or `"splitting_light"` processor to use. If not provided, the latest stable version for the selected `baseProcessor` will be used automatically. See [Splitting Changelog](https://docs.extend.ai/2026-02-09/changelog/splitting/splitting-performance) for more details.
    """

    split_classifications: typing_extensions.Annotated[
        ClassificationsParams, FieldMetadata(alias="splitClassifications")
    ]
    split_rules: typing_extensions.NotRequired[typing_extensions.Annotated[str, FieldMetadata(alias="splitRules")]]
    """
    Custom rules to guide the document splitting process in natural language.
    """

    advanced_options: typing_extensions.NotRequired[
        typing_extensions.Annotated[SplitAdvancedOptionsParams, FieldMetadata(alias="advancedOptions")]
    ]
    """
    Advanced configuration options.
    """

    parse_config: typing_extensions.NotRequired[
        typing_extensions.Annotated[ParseConfigParams, FieldMetadata(alias="parseConfig")]
    ]
    """
    Configuration options for the parsing process.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..core.serialization import FieldMetadata
from ..types.parse_config_advanced_options_excel_parsing_mode import ParseConfigAdvancedOptionsExcelParsingMode
from .page_ranges import PageRangesParams
from .parse_config_advanced_options_return_ocr import ParseConfigAdvancedOptionsReturnOcrParams


class ParseConfigAdvancedOptionsParams(typing_extensions.TypedDict):
    page_rotation_enabled: typing_extensions.NotRequired[
        typing_extensions.Annotated[bool, FieldMetadata(alias="pageRotationEnabled")]
    ]
    """
    Whether to automatically detect and correct page rotation.
    """

    page_ranges: typing_extensions.NotRequired[
        typing_extensions.Annotated[PageRangesParams, FieldMetadata(alias="pageRanges")]
    ]
    excel_parsing_mode: typing_extensions.NotRequired[
        typing_extensions.Annotated[ParseConfigAdvancedOptionsExcelParsingMode, FieldMetadata(alias="excelParsingMode")]
    ]
    """
    Controls how Excel files are parsed.
    
    * `basic`: Fast, deterministic parsing.
    * `advanced`: Enable layout block detection for complex spreadsheets.
    
    For `.xls` files, `basic` mode is always used.
    """

    excel_skip_hidden_content: typing_extensions.NotRequired[
        typing_extensions.Annotated[bool, FieldMetadata(alias="excelSkipHiddenContent")]
    ]
    """
    Whether to exclude hidden rows, columns, and sheets when parsing Excel files.
    """

    vertical_grouping_threshold: typing_extensions.NotRequired[
        typing_extensions.Annotated[float, FieldMetadata(alias="verticalGroupingThreshold")]
    ]
    """
    Multiplier for the Y-axis threshold used to determine if text blocks should be placed on the same line or not (0.1-5.0, default 1.0). Higher values group elements that are further apart vertically. Only applies when the spatial target is set.
    """

    return_ocr: typing_extensions.NotRequired[
        typing_extensions.Annotated[ParseConfigAdvancedOptionsReturnOcrParams, FieldMetadata(alias="returnOcr")]
    ]
    """
    Options for returning raw OCR data in the response.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel


class ParseConfigBlockOptionsTextAgentic(UncheckedBaseModel):
    """
    Options for agentic text processing using VLM-based review and correction.
    """

    enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether to enable agentic text processing for OCR corrections.
    """

    custom_instructions: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="customInstructions")
    ] = pydantic.Field(alias="customInstructions", default=None)
    """
    Custom instructions to guide the agentic text processing.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
